#!/bin/sh
# chkconfig: 35 40 60
# description: javad service
#
# Start Script for Java deamons.
# Adapted for CentOS (Redhat) from the SUSE script indicated below:
#
# Initialization
# Variables to be set in the /etc/sysconfig/javad/$SERVICE config file which is sourced by this script:
#
# USER 	      (= required, username for execution of service)
# VMOPTS      (= optional, java command line args like VM options and system properties)
# PARAMS      (= optional, commandline parameters for the main method)
# CLASSPATH   (= optional, colon separated list of class directories and .jar files)
# MAIN_CLASS  (= required, main class)
# LOG4J       (= optional, path to log4j.xml config file relative to config directory)
# DEBUG	      (= optional, if set debug mode is switched on -> JDWP server is started on port 5005)
# DEBUGPORT   (= optional, set the debug port for the JVM, evaluated only with the DEBUG flag set) 
# DEBUGOPTS   (= optional, set additional parameters for the JDWP server, evaluated only when the DEBUG flag is set )
# JMX	      (= optional, if set jmx mode is switched on -> JMX server is started with ssl and authenticate disabled on port 5006)
# JMXPORT     (= optional, set the jmx port for the JVM, evaluated only with the JMXPORT flag set, by default 5006)
# JMXHOST     (= optional, set the jmx host name where the jmx is listing to, by default jmx host name will be set)
# JMXOPTS     (= optional, set additional parameters for the JMX server, evaluated only when the JMX flag is set ) 
# CONFIGDIR   (= optional, allows to set a non-standard config directory)
# JAVA_HOME   (= optional, allows to set a non-standard java home directory)
# SHOW_ARGS   (= optional, if true, shows the program arguments)
# HOME   (= optional, allows to set the service home directory, by default /var/lib/javad/service/. The application will be started from the home directory)
# APPDIR   (= optional, allows to set the service application directory, by default /usr/share/java/$SERVICE/)

scriptFile=$(readlink -fn $(type -p $0))                   # the absolute, dereferenced path of this script file
scriptDir=$(dirname $scriptFile)                           # absolute path of the script directory
# source init function library (redhat specific, only 'success' and 'failure' functions are used)
if [ -e /etc/redhat-release ]; then
	. /etc/init.d/functions          # Redhat specific, load if it exists
	. $scriptDir/redhat-start-stop.sh
elif [ -e /etc/debian_version ]; then
	. $scriptDir/message-utils.sh     # fallback non-RedHat systems
	. $scriptDir/debian-start-stop.sh
else
	die "unsupported operation system"
fi

function die() {
	echo "*** Error - $1 ***"
	failure
	echo
	exit 1 
}

# read script config parameters

MAIN_CLASS=""
NAME=`basename $0` 
SERVICE=${NAME##javad-}
HOME=/var/lib/javad/$SERVICE
APPDIR=/usr/share/java/$SERVICE
CONFIGDIR=/etc/javad/$SERVICE
[ -e /etc/sysconfig/javad/$SERVICE ] || die "Configuration file /etc/sysconfig/javad/$SERVICE doesn't exist!"

. /etc/sysconfig/javad/$SERVICE

[ -n "$USER" ] || die "USER undefined!" 
[ -n "$MAIN_CLASS" ] || die "MAIN_CLASS undefined!"


# If DEBUG flag is set in the config set the debug parameters
if [ "$DEBUG" == "true" ]; then
	[ -n "$DEBUGPORT" ] || DEBUGPORT=5005
	[ -n "$DEBUGOPTS" ] || DEBUGOPTS="server=y,suspend=n"
	DEBUGPARAMS="-Xdebug -Xnoagent -Djava.compiler=NONE -Xrunjdwp:transport=dt_socket,$DEBUGOPTS,address=$DEBUGPORT -Dlog4j.debug"
fi
# If JMX flag is set in the config set the debug parameters
if [ "$JMX" == "true" ]; then
	[ -n "$JMXPORT" ] || JMXPORT=5005
	[ -n "$JMXOPTS" ] || JMXOPTS="-Dcom.sun.management.jmxremote.authenticate=false -Dcom.sun.management.jmxremote.ssl=false"
	
	JMXPARAMS="-Dcom.sun.management.jmxremote.port=$JMXPORT $JMXOPTS"
	[ -z "$JMXHOST" ] || JMXPARAMS="$JMXPARAMS -Djava.rmi.server.hostname=$JMXHOST"
fi

# set log4j logging system to standard output (will be logged in the standard logfile via pipe)
LOGLEVEL=INFO
if [ "$DEBUG" == "true" ]; then LOGLEVEL=TRACE;fi
LOG4JPARAM="-Dlog4j.rootLogger=\"$LOGLEVEL, stdout\" -Dlog4j.appender.stdout=org.apache.log4j.ConsoleAppender -Dlog4j.appender.stdout.layout=org.apache.log4j.PatternLayout -Dlog4j.appender.stdout.layout.ConversionPattern=\"%d{dd MMM yyyy HH:mm:ss} [%t] %-5p %c{2} - %m%n\""
[ -e $CONFIGDIR/log4j.xml ] && LOG4JPARAM="-Dlog4j.configuration=file:$CONFIGDIR/log4j.xml"
[ -e $CONFIGDIR/log4j.properties ] && LOG4JPARAM="-Dlog4j.configuration=file:$CONFIGDIR/log4j.properties"

# If specified set log4j logging system configuration to the specified file
[ -z "$LOG4J" ] || LOG4JPARAM="-Dlog4j.configuration=file:$LOG4J"

# Build classpath
if [ -e $APPDIR/*.jar ]; then 
	CP_FILE=`ls $APPDIR/*.jar`
fi
for name in $APPDIR/lib/*.jar ; do
    if [ -n "$CP_FILE" ]; then CP_FILE="$CP_FILE:";fi
    CP_FILE="$CP_FILE$name"
done

[ -z "CP_FILE" ] || CP_PARAM="-cp $CP_FILE"

# Build the command line arguments
ARGS="$VMOPTS $JMXPARAMS $DEBUGPARAMS $LOG4JPARAM $CP_PARAM $MAIN_CLASS $PARAMS"

# if DEBUG flag is set print the java command line arguments to the screen
if [ "$SHOW_ARGS" == "true" ]; then echo $ARGS; fi

# /usr/java is the default under a centos, redhat and suse
if [ -z "$JAVA_HOME" ]; then
 	if [ -d /usr/java/default ]; then 
 		# /usr/java/default is the default under a centos
 		JAVA_HOME=/usr/java/default
 	elif [ -d /usr/lib/java ]; then 
 		#default under debian
 		JAVA_HOME=/usr/lib/java	
 	elif [ -d /usr/share/java ]; then 
 		#default under debian
 		JAVA_HOME=/usr/share/java	
	else
 		die "Could not find default java home location. You have to set JAVA_HOME argument"
 	fi
elif [ -d "$JAVA_HOME" ]; then
	die "java home directory does not exist: $JAVA_HOME."
fi

serviceName="$SERVICE"					   # service name 
serviceUser="$USER"                                        # OS user name for the service
serviceGroup="$USER"                                       # OS group name for the service
serviceLogFile="/var/log/javad/$serviceName.log"               # log file for StdOut/StdErr
maxShutdownTime=200                                         # maximum number of seconds to wait for the daemon to terminate normally
pidFile="/var/run/javad/$SERVICE.pid"		                   # name of PID file (PID = process ID number)
javaCommand="java"                                         # name of the Java launcher without the path
javaExe="$JAVA_HOME/bin/$javaCommand"                      # file name of the Java application launcher executable
#fallback to usr/bin/java
[ -e $javaExe ] || javaExe="/usr/bin/$javaCommand"
javaArgs="$ARGS"				           # arguments for Java launcher
javaCommandLine="$javaExe $javaArgs"                       # command line to start the Java service application
javaCommandLineKeyword="$SERVICE"                       # a keyword that occurs on the commandline, used to detect an already running service process and to distinguish it from others


# Makes the file $1 writable by the group $serviceGroup.
function makeFileWritable {
   local filename="$1"
   touch $filename || return 1
   chgrp $serviceGroup $filename || return 1
   chmod g+w $filename || return 1
   return 0; }

# Returns 0 if the process with PID $1 is running.
function checkProcessIsRunning {
   local pid="$1"
   if [ -z "$pid" -o "$pid" == " " ]; then return 1; fi
   if [ ! -e /proc/$pid ]; then return 1; fi
   return 0; }

# Returns 0 if the process with PID $1 is our Java service process.
function checkProcessIsOurService {
   local pid="$1"
   local cmd="$(ps -p $pid --no-headers -o comm)"
   if [ "$cmd" != "$javaCommand" -a "$cmd" != "$javaCommand.bin" ]; then return 1; fi
   grep -q --binary -F "$javaCommandLineKeyword" /proc/$pid/cmdline
   if [ $? -ne 0 ]; then return 1; fi
   return 0; }

# Returns 0 when the service is running and sets the variable $servicePid to
# the PID.
function getServicePid {
   if [ ! -f $pidFile ]; then return 1; fi
   servicePid="$(<$pidFile)"
   checkProcessIsRunning $servicePid || return 1
   checkProcessIsOurService $servicePid || return 1
   return 0; }

function startServiceProcess {
   do_start || return 1
   echo -n "."
   sleep 1
   getServicePid
   if [ $? -ne 0 ]; then
      echo -ne "\n$serviceName start failed, see logfile."
      return 1
   fi
   #wait until look file was created
   while checkProcessIsRunning $servicePid
   do
		if [ -e /$HOME/.javad-lock ]; then return 0;echo; fi
		echo -n "."
		sleep 1
   done
   echo -ne "\n$serviceName start failed, see logfile."
   return 1; }

function stopServiceProcess {
   do_stop || return 1
   for ((i=0; i<maxShutdownTime; i++)); do
      checkProcessIsRunning $servicePid 0 0
      if [ $? -ne 0 ]; then
         rm -f $pidFile
         return 0
         fi
      sleep 1
      echo -n "."
   done
   echo -e "\n$serviceName did not terminate within $maxShutdownTime seconds!"
   return 1; }

function startService {
   getServicePid
   if [ $? -eq 0 ]; then echo -n "$serviceName is already running"; success; echo; return 0; fi
   echo -n "Starting $serviceName "
   startServiceProcess
   if [ $? -ne 0 ]; then failure 1; echo; return 1; fi
   success; echo
   return 0; }

function stopService {
   getServicePid
   if [ $? -ne 0 ]; then echo -n "$serviceName is not running"; success; echo;  return 0; fi
   echo -n "Stopping $serviceName "
   stopServiceProcess
   if [ $? -ne 0 ]; then failure 1; echo;  return 1; fi
   success; echo
   
   return 0; }

function checkServiceStatus {
   echo -n "Checking for $serviceName:   "
   if getServicePid; then
      success; echo
    else
      failure 3; echo
      fi
   
   return 0; }

function main {
#   rc_reset
   case "$1" in
      start)                                               # starts the Java program as a Linux service
         startService
         ;;
      stop)                                                # stops the Java program service
         stopService
         ;;
      restart)                                             # stops and restarts the service
         stopService && startService
         ;;
      status)                                              # displays the service status
         checkServiceStatus
         ;;
      *)
         echo "Usage: $0 {start|stop|restart|status}"
         exit 1
         ;;
      esac
#   rc_exit;i
 }

# execution of main function 
main $1
